import gi
gi.require_version("Gtk", "3.0")
from gi.repository.Gtk import Label, Box, Grid, PositionType, \
    Orientation, Window, WindowType
from clipboard import clipboard_button


# This module is littered with a concept "transpose".
# It is an operation that maps each row to a column
# and each column to a row:
# [[a, b, c], -> [[a, d],
# [[d, e, f]]     [b, e],
#                 [c, f]]


# value is the thing that becomes a grid item
# header is bool telling if this is a header
# transpose is bool telling if this should be transposed
def grid_item(value, header, transpose):
    if isinstance(value, dict):
        # Flip columns to rows and vice versa
        item = info_grid(value, transpose=(not transpose))
    elif isinstance(value, list):
        item = create_box(value, transpose)
    else:
        item = create_header(value) if header else create_label(value)

    return item


def create_label(text):
    label = Label.new(str(text))
    label.set_xalign(0)
    return label


def create_header(text):
    label = Label.new()
    label.set_markup(f"<b>{text}</b>")
    label.set_xalign(0)
    return label


def create_box(list, transpose=False):
    orientation = Orientation.VERTICAL if transpose \
        else Orientation.HORIZONTAL

    box = Box.new(orientation, 5 if transpose else 10)
    for item in list:
        box.pack_start(create_label(item), False, False, 0)

    return box


# Create a grid view of a dictionary.
# Transpose will map rows to columns and columns to rows.
def info_grid(info, transpose=False):
    grid = Grid.new()
    grid.set_row_spacing(15)
    grid.set_column_spacing(15)
    item_size = {"width": 1, "height": 1}

    # Map headers and fields to grid items
    for i, (k, v) in enumerate([(grid_item(k, True, transpose=transpose),
                                 grid_item(v, False, transpose=transpose))
                               for k, v in info.items()]):
        if transpose:
            top = 0
            left = i
            side = PositionType.BOTTOM
        else:
            top = i
            left = 0
            side = PositionType.RIGHT

        grid.attach(k, left=left, top=top, **item_size)
        grid.attach_next_to(v, k, side, **item_size)

    return grid


# Takes in a title-string and info-dictionary.
# Returns a window.
def info_window(title, info):
    box = Box(orientation=Orientation.VERTICAL, spacing=15)
    box.pack_start(info_grid(info), True, True, 0)
    box.add(clipboard_button(info))

    window = Window.new(WindowType.TOPLEVEL)
    window.set_title(title)
    window.set_border_width(10)

    window.add(box)

    window.show_all()
